PROC GLOBAL

{ --------------------------------------------------------------------------- }

set explicit;
{ Working variables }
array wenter(50); { Working var recording entry of individuals }
alpha(26) alphavar;
alpha(26) alphanew;

numeric zscoeff zsr zsb1 zsb2 zsb3 zsb4 zsb5 zsdz zst
  zsdzp zsfx zsll zsul zsa zsknot zsi zsj zsk zsl
  zsz zsc zsy1 zsy2 zsy3 zssd1 zssd2 zshta
  zspmed zsv zshac zshap zshaz zswap zswaz zswac zshtw
  zswhp zswhz zswhc fhaz fwaz fwhz zsflag leapday x
  xdays1 y xdays2 vcheckx addmode qmark lx l totlen
  aok alphsize a maxmem maxovc hhnum minage maxage minab
  chhlth chlabl chlabu mindi cln err i dd ss
  di lev grd numovc j xline xwomen xchild
  ddw diwm udob ldob t ldobfc udobfc ldoblc udoblc
  dich hwsex hwage hwerr birthd birthm
  birthy age xweight xheight
  ldofm udofm minam;

{ --------------------------------------------------------------------------- }

function valid(xvar);
  valid = (!special(xvar) and xvar < 96)
end;

function NAtoZero(xvar);
  if xvar = notappl then
    xvar = 0
  endif;
  NAtoZero = xvar;
end;

{ --------------------------------------------------------------------------- }

{ Function to check all possible combinations of day, month and year }
function vdvalid(vcheckd,vcheckm,vchecky);
  vdvalid = 0;
  { Check day, month and year are consistent }
  recode    vchecky   : vcheckm               : vcheckd    => vcheckx; { !!! }
  2000-2006,9996-9999 : 1,3,5,7,8,10,12,96-99 : 1-31,96-99 => 0;
  2000-2006,9996-9999 : 4,6,9,11              : 1-30,96-99 => 0;
  2000-2006,9996-9999 : 2                     : 1-28,96-99 => 0;
  2000,2004           : 2                     : 29         => 0;       { leap year }
              notappl : notappl               : 66         => 0;
              notappl : notappl               : 44         => 0;
              notappl : notappl               : 0          => 0;
                      :                       :            => 1;
  endrecode;
  if vcheckx then
    vdvalid = 1;
  endif;
end;

{ Check vaccination date is before date of interview }
function vdoi(vcheckd,vcheckm,vchecky);
  vdoi = 0;
  if (vcheckd < 44 and vcheckd > 0) and valid(vcheckm) and vchecky < 9996 then
    if (cmcode(vcheckm,vchecky) > cmcode(UF8M,UF8Y)) |
        (cmcode(vcheckm,vchecky) = cmcode(UF8M,UF8Y) and vcheckd > UF8D) then
      vdoi = 1;
    endif;
  elseif valid(vcheckm) and vchecky < 9996 then
    if (cmcode(vcheckm,vchecky) > cmcode(UF8M,UF8Y)) then
      vdoi = 1;
    endif;
  elseif vchecky < 9996 then
    if vchecky > UF8Y then
      vdoi = 1;
    endif;
  endif;
end;

{ Check vaccination date is after date of birth }
function vdob(vcheckd,vcheckm,vchecky);
  vdob = 0;
  if valid(UF10D) and valid(UF10M) and UF10Y < 9996 then
    if vcheckd < 44 and valid(vcheckm) and vchecky < 9996 then
      if cmcode(vcheckm,vchecky) < cmcode(UF10M,UF10Y) | (cmcode(vcheckm,vchecky) = cmcode(UF10M,UF10Y) and vcheckd < UF10D) then
        vdob = 1;
      endif;
    elseif valid(vcheckm) and vchecky < 9996 and cmcode(vcheckm,vchecky) < cmcode(UF10M,UF10Y) then
      vdob = 1;
    elseif vchecky < 9996 and vchecky < UF10Y then
      vdob = 1;
    endif;
  elseif valid(UF10M) and UF10Y < 9996 then
    if valid(vcheckm) and vchecky < 9996 and cmcode(vcheckm,vchecky) < cmcode(UF10M,UF10Y) then
      vdob = 1;
    elseif vchecky < 9996 and vchecky < UF10Y then
      vdob = 1;
    endif;
  elseif UF10Y < 9996 then
    if vchecky < 9996 and vchecky < UF10Y then
      vdob = 1;
    endif;
  endif;
end;


function endmess();
  { Returns true if response is REVIEW }
  endmess = (addmode and
             accept("i ",
                    " ",
                    "   ") <> 2);
end;

{ Function to check entry of alpha string variables }
function alphachk(alpha(26) alphamask);
  { first remove all blanks, and duplicate question marks }
  alphanew = "";
  qmark = 0;
  lx = 1;
  do l = 1 while l <= length(strip(alphavar))
    if alphavar[l:1] <> " " and (alphavar[l:1] <> "?" or not qmark) then
      alphanew[lx:1] = alphavar[l:1];
      lx = lx + 1;
      if alphavar[l:1] = "?" then
        qmark = 1;
      endif;
    endif;
  enddo;
  { aok = 0 - string is bad, aok = 1 - string is good }
  totlen = length(strip(alphamask));
  aok = 0;           { assume string is bad - for empty string }
  alphsize = length(strip(alphanew));
  if alphsize then   { not empty string }
    { check for "?" as only character }
    aok = (alphanew[1:1] = "?" and alphsize = 1);
    if !aok then     { if not a single "?", check string }
      aok = 1;       { now assume string is good until we know otherwise }
      l = 1;         { l is position in string of acceptable values }
      a = 1;         { a is position in input string }
      { loop while the string is still good and there are more letters }
      while aok and a <= alphsize do
        aok = pos(alphanew[a:1],alphamask[l:totlen-l+1]);
        if aok then     { letter is acceptable }
          l = l + aok;  { increment l to position after letter in string of acceptable values }
          a = a + 1     { increment a to next letter in input string }
        endif;
      enddo;
    endif;
  endif;
  alphachk = (!aok);    { alphachk = 0 - if string is good, = 1 - if string is bad }
  { now expand variables }
  if aok then
    do l = 1 while l <= length(strip(alphamask))
      if pos(alphamask[l:1],alphanew) then
        alphavar[l:1] = alphamask[l:1]
      elseif pos("?",alphanew) then
	alphavar[l:1] = "?"
      else
	alphavar[l:1] = " "
      endif;
    enddo;
  endif;
end;



PROC FL_ENTRY
preproc
maxmem = 50;      { maximum number of household members }
maxovc = 50;      { maximum number of OVCs }
hhnum  = 0;       { last household number used }
minage = 15;      { minimum age of eligible women }
maxage = 49;      { maximum age of eligible women }
minab  = 144;     { minimum age at birth in months = 12 years }
minam  = 156;     { minimum age at first marriage = 13 years }
chhlth = 4;       { maximum age at which child is included in child health module - excludes exact age 5 }
chlabl = 5;       { minimum age at which child is included in child labor module - includes exact age 5 }
chlabu = 14;      { maximum age at which child is included in child labor module - excludes exact age 15 }
mindi = 20050401; { replace "20050401" with earliest interview date in yyyymmdd format }
addmode = (demode() = ADD);
cln=tonumber(sysparm());
recode cln  => err;
    001-100 => 0;    { Give list of valid cluster numbers }
            => 1;
endrecode;
if err then
  errmsg(0010);
  endlevel;
endif;

PROC HH
{ *** Level 1 *** }
preproc
  { Initialization prior to the entry of a household questionnaire }

  { Initialize the working woman's line number
    and the flags for the entry of eligible women }
  i = 1;
  while i <= maxmem do
    wenter(i) = 0;
    i = i + 1;
  enddo;

PROC FORMHH
if HH9 <> 1 then
  if endmess() then
    reenter HH16
  endif;
  endlevel
endif;

PROC HH1
if HH1 <> cln then
  errmsg(0010);
  reenter
endif;

PROC HH2
if addmode and HH2 < hhnum then
  errmsg(0012);
endif;
hhnum = HH2;

PROC HH5Y
recode HH5Y  :     HH5M        : HH5D => err;
   2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
   2005-2006 : 4,6,9,11        : 1-30 => 0;
   2005-2006 : 2               : 1-28 => 0;
             :                 :      => 1;
endrecode;
if err then
  errmsg(0016);
  reenter
endif;
dd = (HH5Y*100 + HH5M)*100 + HH5D;
ss = sysdate("yyyymmdd");
if dd >= ss or dd < mindi then
  errmsg(0016);
  reenter
endif;
di = cmcode(HH5M,HH5Y);

PROC HH6
recode HH1  : HH6 => err;
    1-3    : 1  => 0;
    4-5    : 2  => 0;
    6-7    : 1  => 0;
    8-9    : 2  => 0;
    10     : 1  => 0;
    11-12  : 2  => 0;
    13-18  : 1  => 0;
    19     : 2  => 0;
    20-26  : 1  => 0;
    27     : 2  => 0;
    28-29  : 1  => 0;
    30     : 2  => 0;
    31     : 1  => 0;
    32-33  : 2  => 0;
    34-36  : 1  => 0;
    37     : 2  => 0;
    38     : 1  => 0;
    39-40  : 2  => 0;
    41-42  : 1  => 0;
    43     : 2  => 0;
    44     : 1  => 0;
    45     : 2  => 0;
    46-49  : 1  => 0;
    50     : 2  => 0;
    51-54  : 1  => 0;
    55-57  : 2  => 0;
    58-59  : 1  => 0;
    60     : 2  => 0;
    61-63  : 1  => 0;
    64-65  : 2  => 0;
    66-67  : 1  => 0;
    68     : 2  => 0;
    69     : 1  => 0;
    70-71  : 2  => 0;
    72-73  : 1  => 0;
    74     : 2  => 0;
    75     : 1  => 0;
    76-77  : 2  => 0;
    78-81  : 1  => 0;
    82     : 2  => 0;
    83     : 1  => 0;
    84     : 2  => 0;
    85-86  : 1  => 0;
    87     : 2  => 0;
    88     : 1  => 0;
    89     : 2  => 0;
    90     : 1  => 0;
    91-92  : 2  => 0;
    93     : 1  => 0;
    94     : 2  => 0;
    95-100 : 1  => 0;
           :    => 1;
endrecode;
if err then
  errmsg(0011);
  reenter
endif;

PROC HH7
recode HH1  : HH7 => err;
    1-5    : 1  => 0;
    6-9    : 5  => 0;
    10-12  : 7  => 0;
    13-19  : 12 => 0;
    20-27  : 14 => 0;
    28-30  : 18 => 0;
    31-33  : 21 => 0;
    34-37  : 23 => 0;
    38-40  : 26 => 0;
    41-43  : 32 => 0;
    44-45  : 35 => 0;
    46-50  : 44 => 0;
    51-57  : 46 => 0;
    58-60  : 48 => 0;
    61-65  : 51 => 0;
    66-68  : 53 => 0;
    69-71  : 56 => 0;
    72-74  : 59 => 0;
    75-77  : 61 => 0;
    78-82  : 63 => 0;
    83-84  : 65 => 0;
    85-87  : 68 => 0;
    88-89  : 71 => 0;
    90-92  : 73 => 0;
    93-94  : 74 => 0;
    95-100 : 80 => 0;
           :    => 1;
endrecode;
if err then
  errmsg(0011);
  reenter
endif;

PROC HH9
if HH9 <> 1 then
  skip to HH16
endif;

PROC HH12
if HH12 > HH11 then
  errmsg(0015);
  reenter
endif;

PROC HH13
if HH13 > HH12 then
  errmsg(0014);
  reenter
endif;

PROC HH14
if HH12+HH14 > HH11 then
  errmsg(0015);
  reenter
endif;

PROC HH15
if HH15 > HH14 then
  errmsg(0013);
  reenter
endif;

PROC FORMHL


PROC FORMHL_ROSTER {NEW}
i = 1;
while i <= noccurs(FORMHL_ROSTER) do
  { Check that HH member identified by HL10 could be child's mother }
  if HL10(i) >= 1 then
    if HL4(HL10(i)) <> 2 or HL5(HL10(i)) < HL5(i)+(minab/12) then
      errmsg(0164,i,HL4(HL10(i)),HL5(HL10(i)),"","HL10",HL10(i));
    endif;
  endif;
  { Check that HH member identified by HL12 could be child's father }
  if HL12(i) >= 1 then
    if HL4(HL12(i)) <> 1 or HL5(HL12(i)) < HL5(i)+(minab/12) then
      errmsg(0164,i,HL4(HL12(i)),HL5(HL12(i)),"","HL12",HL12(i));
    endif;
  endif;
  { Check the spouse of head of household is of opposite sex }
  if HL3(i) = 2 and HL4(i) = HL4(1) then
    errmsg(0142,i);
  { Check children of head of household are at least 12 years younger }
  elseif HL3(i) = 3 and HL5(1) < HL5(i)+(minab/12) then
    errmsg(0143,minab/12,i);
  { Check grandchildren of head of household are at least 24 years younger }
  elseif HL3(i) = 5 and HL5(1) < HL5(i)+(minab/6) then
    errmsg(0145,minab/6,i);
  { Check parents of head of household are at least 12 years older }
  elseif HL3(i) = 6 and HL5(1)+(minab/12) > HL5(i) then
    errmsg(0146,minab/12,i);
  endif;
  i = i + 1
enddo;

PROC HL1
preproc
if noccurs(FORMHL_ROSTER) >= HH11 then
  endgroup
endif;

i = 1;
while i <= maxmem do
  if i <= HH11 then
    HL1(i) = i
  else
    HL1(i) = notappl
  endif;
  i = i + 1
enddo;
PROC HL3 {NEW}
{ Head of household must be on first line }
if (HL3 = 1) <=> (HL1 <> 1) then
  errmsg(0141);
endif;

PROC HL6
preproc
if not (HL4 = 2 and HL5 in minage:maxage) then
  HL6 = 0;
  noinput;
endif;

postproc
if HL4 = 2 and HL5 in minage:maxage then
  if HL6 <> HL1 then
    errmsg(0101);
    reenter
  endif;
else
  if HL6 <> 0 then
    errmsg(0102);
    reenter
  endif;
endif;

PROC HL7
preproc
if not (HL5 in chlabl:chlabu) then
  HL7 = 0;
  noinput;
endif;

postproc
if HL5 in chlabl:chlabu then
  if HL7 = 0 or HL7 = HL1 then
    errmsg(0121);
    reenter
  elseif HL7 > HH11 then
    errmsg(0120,HL8,HL11);
    reenter
  endif;
else
  if HL7 <> 0 then
    errmsg(0122);
    reenter
  endif;
endif;

PROC HL8
preproc
if HL5 > chhlth then
  HL8 = 0;
  noinput;
endif;

postproc
if HL5 <= chhlth then
  if HL8 = 0 or HL8 = HL1 then
    errmsg(0131);
    reenter
  elseif HL8 > HH11 then
    errmsg(0130,HL8,HL11);
    reenter
  endif;
else
  if HL8 <> 0 then
    errmsg(0132);
    reenter
  endif;
endif;

PROC HL9
preproc
if HL5 > 17 then
  skip to next HL1
endif;

postproc
if HL9 <> 1 then
  skip to HL11
endif;

PROC HL10
if HL10 >= 1 then {NEW}
  if valid(HL10) and HL10 > HH11 then
    errmsg(0161,"i",HL10,HH11);
    reenter;
  elseif HL10 = HL1 then
    errmsg(0162,"i",HL10,HL1);
    reenter;
  elseif HL7 <> 0 then
    if HL10 <> HL7 then
      errmsg(0163,HL10,"HL7",HL7);
    endif;
  elseif HL8 <> 0 then
    if HL10 <> HL8 then
      errmsg(0163,HL10,"HL8",HL8);
    endif;
  endif;
  skip to HL11
endif;

PROC HL11
if HL11 <> 1 then
  skip to next HL1
endif;

PROC HL12
if HL12 >= 1 then {NEW}
  if valid(HL12) and HL12 > HH11 then
    errmsg(0161,"",HL12,HH11);
    reenter;
  elseif HL12 = HL1 then
    errmsg(0162,"",HL12,HL1);
    reenter;
  endif;
  skip to next HL1
endif;

PROC TOHL6 {NEW}
if TOHL6 <> count(FORMHL_ROSTER where HL6 > 0) then
  errmsg(0110,"i 15-49");
  reenter;
elseif TOHL6 <> HH12 then
  errmsg(0111,"i 15-49",TOHL6,"HH12",HH12);
  reenter;
endif;

PROC TOHL7 {NEW}
if TOHL7 <> count(FORMHL_ROSTER where HL7 > 0) then
  errmsg(0110,"i 5-14");
  reenter;
endif;

PROC TOHL8 {NEW}
if TOHL8 <> count(FORMHL_ROSTER where HL8 > 0) then
  errmsg(0110,"i  5 i");
  reenter;
elseif TOHL8 <> HH14 then
  errmsg(0111,"i  5 i",TOHL8,"HH14",HH14);
  reenter;
endif;

PROC FORMED


PROC ED1
preproc
if noccurs(FORMED_ROSTER) >= HH11 then
  endgroup
endif;


i = 1;
while i <= maxmem do
  if i <= HH11 then
    ED1(i) = i
  else
    ED1(i) = notappl
  endif;
  i = i + 1
enddo;
PROC ED2
preproc
if HL5(ED1) < 5 then
  skip to next ED1
endif;

postproc
if ED2 <> 1 then
  skip to next ED1
endif;

PROC ED3A
postproc
if HL5(ED1) < 9 and ED3A in 2, 3 then {NEW}
  errmsg(0100,HL5(ED1));
  reenter;
endif;
if HL5(ED1) < 16 and ED3A = 3 then {NEW}
  errmsg(0100,HL5(ED1));
  reenter;
endif;
PROC ED3B
lev = ED3A;
grd = ED3B;
recode lev  : grd         => err; { Fix maximum grade for each possible level }
            : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
          0 : 0-6         => 0;   { Preschool}
          1 : 0-6         => 0;   { Primary }
          2 : 0-12        => 0;   { Secondary }
          3 : 0-6         => 0;   { Higher }
          6 : 0-6         => 0;   { Non-standard curriculum }
          8 :             => 0;   { DK level }
          9 :             => 0;   { Missing level }
            :             => 1;
endrecode;
if err then
  errmsg(0090);
  { reenter }
endif;

PROC ED4
preproc
if HL5(ED1) > 24 then
  skip to next ED1
endif;

postproc
if ED4 <> 1 then
  skip to ED7
endif;

PROC ED6A
if ED6A <= 3 and ED3A <= 3 and ED6A > ED3A then {NEW}
  errmsg(0091,ED6A,ED3A);
  reenter;
endif;

PROC ED6B
lev = ED6A;
grd = ED6B;
recode  lev : grd         => err; { Fix maximum grade for each possible level }
            : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
          0 : 1-6         => 0;   { Preschool}
          1 : 1-6         => 0;   { Primary }
          2 : 1-12        => 0;   { Secondary }
          3 : 1-6         => 0;   { Higher }
          6 : 1-6         => 0;   { Non-standard curriculum }
          8 :             => 0;   { DK level }
          9 :             => 0;   { Missing level }
            :             => 1;
endrecode;
if err then
  errmsg(0090);
  reenter
endif;

{ Compare current grade and level to max grade and level }
if (ED3A = ED6A) and valid(ED6B) and valid(ED3B) and ED6B > ED3B+1 then {NEW}
  errmsg(0092,ED6B,ED3B);
endif;

PROC ED7
if ED7 <> 1 then
  skip to next ED1
endif;

PROC ED8A
if ED8A <= 3 and ED3A <= 3 and ED8A > ED3A then {NEW}
  errmsg(0093,ED8A,ED3A);
  reenter;
endif;

PROC ED8B
lev = ED8A;
grd = ED8B;
recode  lev : grd         => err; { Fix maximum grade for each possible level }
            : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
          0 : 1-6         => 0;   { Preschool}
          1 : 1-6         => 0;   { Primary }
          2 : 1-12        => 0;   { Secondary }
          3 : 1-6         => 0;   { Higher }
          6 : 1-6         => 0;   { Non-standard curriculum }
          8 :             => 0;   { DK level }
          9 :             => 0;   { Missing level }
            :             => 1;
endrecode;
if err then
  errmsg(0090);
  reenter
endif;

{ Compare last year's grade and level to max grade and level }
if (ED3A = ED8A) and valid(ED8B) and valid(ED3B) and ED8B > ED3B+1 then {NEW}
  errmsg(0094,ED8B,ED3B);
endif;

PROC WS1
if WS1 in 11, 12 then
  skip to WS5
elseif WS1 in 13:81,96,99 then
  skip to WS3
{ elseif WS1 = 91 then { bottled water }
  skip to WS2 }
endif;

PROC WS2
if WS2 in 11, 12 then
  skip to WS5
endif;

PROC WS3
if WS3 = 995 then
  skip to WS5
elseif WS3 < 995 and WS3%10 <> 0 and WS3%10 <> 5 then
  errmsg(9993)
endif;

PROC WS5
if WS5 <> 1 then
  skip to WS7
endif;

PROC WS6
alphavar = $;
if alphachk("ABCDEFXZ") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "Don't Know" is only response }
if pos("Z",$) then
  if alphsize > 1 then
    errmsg(9996);
    reenter
  endif;
endif;

PROC WS7
if WS7 = 95 then
  endgroup
endif;

PROC WS8
if WS8 <> 1 then
  endgroup
endif;

PROC HC6
if HC6 in 1:4 then
  skip to HC8
endif;

PROC HC7
if HC7 = 3 or HC7 = 6 then
  skip to HC8
endif;

PROC FORMCL

PROC CL1
preproc
if noccurs(FORMCL_ROSTER) >= HH11 then
  endgroup
endif;

i = 1;
while i <= maxmem do
  if i <= HH11 then
    CL1(i) = i
  else
    CL1(i) = notappl
  endif;
  i = i + 1
enddo;

postproc
if not HL5(CL1) in 5:14 then
  skip to next CL1
endif;

PROC CL3
if CL3 >= 3 then
  skip to CL5
endif;

PROC CL4
skip to CL6;

PROC CL6
if CL6 <> 1 then
  skip to CL8
endif;

PROC CL7
if NAtoZero(CL4) + NAtoZero(CL7) > 80 then {NEW}
  errmsg(0201);
endif;

PROC CL8
if CL8 <> 1 then
  skip to next CL1
endif;

PROC CL9
if NAtoZero(CL4) + NAtoZero(CL7) + NAtoZero(CL9) > 80 then {NEW}
  errmsg(0201);
endif;

PROC FORMCD
preproc
if count(FORMHL_ROSTER where HL5 in  2:14) = 0 then
  endgroup
endif;

PROC CD7 {NEW}
if CD7 <> count(FORMHL_ROSTER where HL5 in 2:14) then
  errmsg(0301);
endif;

PROC CD9 {NEW}
if CD9 > CD7 then
  errmsg(0302,CD9,CD7);
elseif CD7 = 1 and CD9 <> 1 then
  errmsg(0303);
endif;

PROC CD11 {NEW}
if valid(CD11) then
  if CD11 > HH11 then
    errmsg(0304,CD11,HH11);
    reenter;
  elseif  not (HL5(CD11) in 2:14) then
    errmsg(0305,HL5(CD11));
    reenter;
  endif;
endif;

PROC WMCH
{ *** Level 2 *** }
preproc
  { Initialization prior to an individual questionnaire }
  xline = 0;
  { Find the next eligible woman to enter }
  i = 1;
  while i <= HH11 do
    if HL6(i) and wenter(i) = 0 then
      xline = i;
      i = HH11;  { Having found a woman exit the while-do loop }
    endif;
    i = i + 1;
  enddo;
  if !xline then
    { Find the next eligible child to enter }
    i = 1;
    while i <= HH11 do
      if HL8(i) and wenter(i) = 0 then
        xline = i;
        i = HH11;  { Having found a child exit the while-do loop }
      endif;
      i = i + 1;
    enddo;
  endif;
  { Quit level 2 if no individual questionnaires to enter }
  if !xline then
    endlevel
  endif;

postproc
  { Updating of information after completion of an individual questionnaire }

  { Update the entry flag for individual questionnaire just entered }
  if xline then
    if HL6(xline) <> 0 then
      wenter(xline) = 1
    elseif HL8(xline) <> 0 then
      wenter(xline) = 2
    endif;
  endif;

  { Check the number of individual questionnaires entered and
    quit level 2 if no more to enter }
  xwomen = 0;
  do varying i = 1 until i > HH11
    if wenter(i) = 1 then
      xwomen = xwomen + 1
    endif;
  enddo;
  xchild = 0;
  do varying i = 1 until i > HH11
    if wenter(i) = 2 then
      xchild = xchild + 1
    endif;
  enddo;

  if xwomen = HH12 and xchild = HH14 then
    endlevel
  endif;

PROC LN
preproc
LN = xline;

postproc
if HL8(LN) <> 0 then
  skip to FORMCH
endif;

PROC WM1
preproc
WM1 = HH1;
WM2 = HH2;

PROC WM4
preproc
if addmode then
  display(concat("  i ",edit("99", LN)));
endif;

postproc
if WM4 <> LN then
  errmsg(1000,"i",LN);
  reenter
endif;


PROC WM6Y
recode WM6Y   :     WM6M        : WM6D => err;
    2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
    2005-2006 : 4,6,9,11        : 1-30 => 0;
    2005-2006 : 2               : 1-28 => 0;
              :                 :      => 1;
endrecode;
if err then
  errmsg(0016);
  reenter
endif;
dd = (HH5Y*100 + HH5M)*100 + HH5D;
ddw= (WM6Y*100 + WM6M)*100 + WM6D;
ss = sysdate("yyyymmdd");
if ddw >= ss or ddw < dd then
  errmsg(0016);
  reenter
endif;
diwm = cmcode(WM6M,WM6Y);

PROC WM7
if WM7 <> 1 then
  if endmess() then
    reenter
  endif;
  endlevel
endif;

PROC WM8Y
udob = diwm - minage*12;
ldob = diwm - maxage*12 - 11;
if WM8Y < 9997 then
  ldob = setlb(WM8M,WM8Y,0);
  udob = setub(WM8M,WM8Y,9999);
  if udob < diwm - maxage*12 - 11 or ldob > diwm - minage*12 then
    errmsg(1001);
  { reenter }
  endif;
endif;

PROC WM9
if WM9 <> HL5(WM4) then
  errmsg(1002,WM9,HL5(LN));
endif;
{ Lower bound of CMC date of birth }
t = adjlba(ldob,udob,diwm,diwm,WM9);
if t < 0 then
  errmsg(1003);
{ reenter }
else
  ldob = t
endif;
{ Upper bound of CMC date of birth }
t = adjuba(ldob,udob,diwm,diwm,WM9);
if t < 0 then
  errmsg(1003);
{ reenter }
else
  udob = t
endif;

PROC WM10
if WM10 <> ED2(LN) and ED3(LN) <> 0 then
 errmsg(1011,ED2(LN),WM10);
 endif;
 if $ <> 1 then
  endgroup
endif;

PROC WM11
if WM11 <> ED3A(LN) then {NEW}
  errmsg(1012,ED3A(LN),WM11);
endif;

PROC WM12
if WM12 <> ED3B(LN) then {NEW}
  errmsg(1013,ED3B(LN),WM12);
endif;
lev = WM11;
grd = WM12;
recode lev  : grd         => err; { Fix maximum grade for each possible level }
            : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
          0 : 0-6         => 0;   { Preschool}
          1 : 0-6         => 0;   { Primary }
          2 : 0-12        => 0;   { Secondary }
          3 : 0-6         => 0;   { Higher }
          6 : 0-6         => 0;   { Non-standard curriculum }
          8 :             => 0;   { DK level }
          9 :             => 0;   { Missing level }
            :             => 1;
endrecode;
if err then
  errmsg(1014);
  { reenter }
endif;
{ WM13 Filter }
if WM11 in 2,3 then
  endgroup
endif;

PROC CM1
if CM1 <> 1 then
  skip to FORMMA
endif;

PROC CM2AY
ldobfc = ldob + minab;
udobfc = diwm;
if CM2AY < 9997 then
  ldobfc = setlb(CM2AM,CM2AY,0);
  udobfc = setub(CM2AM,CM2AY,9999);
  if udobfc < ldob + minab then
    errmsg(1101,int(minab/12));
  elseif ldobfc > diwm then
    errmsg(1102);
  endif;
  skip to CM3
endif;

PROC CM2B {NEW}
if WM9 - CM2B < (minab/12) then
  errmsg(1103,minab/12);
endif;

PROC CM3
if CM3 <> 1 then
  skip to CM5
endif;

PROC CM4B
{ Check for at least one child living at home - otherwise CM3 must be 2 }
if CM4A = 0 and CM4B = 0 then
  errmsg(1110);
  reenter
endif;

PROC CM5
if CM5 <> 1 then
  skip to CM7
endif;

PROC CM6B
{ Check for at least one child living away - otherwise CM5 must be 2 }
if CM6A = 0 and CM6B = 0 then
  errmsg(1110);
  reenter
endif;

PROC CM7
if CM7 <> 1 then
  skip to CM9
endif;

PROC CM8B
{ Check for at least one child who died - otherwise CM7 must be 2 }
if CM8A = 0 and CM8B = 0 then
  errmsg(1110);
  reenter
endif;

PROC CM9
{ Count up total children ever born }
x = NAtoZero(CM4A)+NAtoZero(CM4B)+
    NAtoZero(CM6A)+NAtoZero(CM6B)+
    NAtoZero(CM8A)+NAtoZero(CM8B);
{ Check count of children ever born with total }
if x <> CM9 then
  errmsg(1111);
  reenter;
elseif CM9 = 0 then
  errmsg(1110);
  reenter;
endif;

PROC CM11Y
ldoblc = ldobfc;
udoblc = diwm;
if CM11Y < 9997 then
  ldoblc = setlb(CM11M,CM11Y,0);
  udoblc = setub(CM11M,CM11Y,9999);
  if udoblc < ldob + minab then
    errmsg(1121,int(minab/12));
  elseif ldoblc > diwm then
    errmsg(1122);
  endif;
endif;

if CM9 = 1 then
  if (CM11Y <> CM2AY or CM11M <> CM2AM or CM11D <> CM2AD) then
    errmsg(1123);
  endif;
else
  if ldobfc > udoblc then
    errmsg(1124);
  endif;
endif;

PROC CM12
{ Check if birth in last 24 months }
{ Note use of parentheses for correct check }
if CM11Y = WM6Y or CM11Y = WM6Y-1 or (CM11Y = WM6Y-2 and
 ((CM11M > WM6M and CM11M <= 12)  or (CM11M = WM6M and
   CM11D > WM6D and CM11D <= 31))) then
  if CM12 <> "Y" then
    errmsg(1131);
    reenter
  endif;
else
  if CM12 <> "N" then
    errmsg(1132);
    reenter
  endif;
endif;

if CM12 = "N" then
  skip to FORMMA
endif;

PROC MN2
alphavar = $;
if alphachk("ABFGHXY") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "No one" is only response }
if pos("Y",MN2) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
  else
    skip to MN6A
  endif;
endif;

PROC MN3A
if $ <> 1 then
  skip to MN3B
endif;
PROC MN3B
if $ <> 1 then
  skip to MN3C
endif;
PROC MN3C
if $ <> 1 then
  skip to MN3D
endif;
PROC MN3D
if $ <> 1 then
  skip to MN3E
endif;
PROC MN3E
if $ <> 1 then
  skip to MN3F
endif;
PROC MN3F
if $ <> 1 then
  skip to MN4
endif;
PROC MN3G
alphavar = $;
if alphachk("ABC") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;


PROC MN5
if MN5 <> 1 then
  skip to MN6A
endif;

PROC MN6A
if MN6A <> 1 then
  skip to MN7
endif;

PROC MN7
alphavar = $;
if alphachk("ABFGHXY") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "No one" is only response }
if pos("Y",MN7) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
  endif;
endif;

PROC MN8
if $ in 11,12 then
  skip to MN9
endif;
PROC MN10
if MN10 <> 1 then
  skip to MN12
endif;

PROC MN11
{ Check special answers }
if (MN11A = 9 <=> MN11 < 9.997) then
  errmsg(1301);
  reenter
endif;

PROC MN12
if MN12 <> 1 then
  endgroup
endif;

PROC MN13N
recode MN13U  : MN13N       => err;
          0   : 0           => 0;   { Immediately }
          1   : 0-23,99     => 0;   { Hours }
          2   : 1-90,99     => 0;   { Days }
          9   : 98-99       => 0;   { DK, Missing }
              :             => 1;
endrecode;
if err then
  errmsg(9992);
  reenter;
endif;

PROC MA1
if MA1 = 3 then
  skip to MA3
endif;

PROC MA3
preproc
{ This instruction is used to handle skip from MA2 if polygyny questions are not used
  and from MA2B if polygyny questions are used }
if MA1 in 1, 2 then
  skip to MA5
endif;

postproc
if MA3 = 3 then
  endgroup
endif;

PROC MA6Y
{ Check woman at least 4 when married (minam=min. age at marriage) } {NEW}
if MA6Y < 9996 and WM8Y < 9996 then
  if valid(MA6M) and valid(WM8M) then
    if cmcode(MA6M,MA6Y) - cmcode(WM8M,WM8Y) < 0 then
      errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,0);
    elseif cmcode(MA6M,MA6Y) - cmcode(WM8M,WM8Y) < minam then
      errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,minam/12);
    endif;
  else
    if MA6Y - WM8Y < 0 then
      errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,0);
    elseif MA6Y - WM8Y < int(minam/12)-1 then
      errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,minam/12);
    endif;
  endif;
endif;
{ MA7 Filter }
if MA6Y < 9997 and MA6M < 97 then
  endgroup
endif;

PROC MA8
{ Check date of first marriage consistent with age at first marriage } {NEW --- please check TC!}
if MA6Y < 9997 and valid(MA8) then
  ldofm = setlb(1,MA6Y,0);
  udofm = setub(12,MA6Y,9999);
  { Lower bound of CMC date of first marriage }
  t = adjlba(ldob,udob,ldofm,udofm,MA8);
  if t < 0 then
    errmsg(1403,MA8,MA6M,MA6Y,WM8M,WM8Y);
  endif;
  { Upper bound of CMC date of first marriage }
  t = adjuba(ldob,udob,ldofm,udofm,MA8);
  if t < 0 then
    errmsg(1403,MA8,MA6M,MA6Y,WM8M,WM8Y);
  endif;
endif;
{ Check age at first marriage against age }
if valid(MA8) and MA8 > WM9 then {NEW}
  errmsg(1404,MA8,WM9);
endif;

PROC CP0A
if $ <> 1 then
  skip to CP1
endif;
PROC CP0B
alphavar = $;
if alphachk("ABCDEFGXZ") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "Don't remember" is only response }
if pos("Z",CP0B) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
  endif;
endif;
PROC CP1
if CP1  <> 1 then
  skip to CP2
endif;

PROC CP1A
skip to CP4A

PROC CP2
if CP2 <> 1 then
  skip to CP4A
endif;

PROC CP3
alphavar = $;
if alphachk("ABCDEFGHIJKLMX") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;

PROC CP4A
if CP4A  = 3 then
  endgroup 
elseif CP4A <> 1 then 
  skip to CP4E
endif;
PROC CP4CN
recode CP4CU  : CP4CN       => err;
          1,2 : 0-90,99     => 0;   { Months, Years }
          9   : 93-96,98,99 => 0;   { Special, DK, Missing }
              :             => 1;
endrecode;
if err then
  errmsg(9992);
  reenter
endif;

if CP4CU = 9 and CP4CN = 94 then
  endgroup
endif;

PROC CP4E
preproc
if CP1 = 1 then
  endgroup
endif; 

PROC FORMHA
if endmess() then
  if HA1 <> 1 then
    reenter HA1
    else
   if HA20 <> 1 then
    reenter HA20
    else
    reenter HA21
  endif;
  endif;
endif;
endlevel;
PROC HA1
if HA1 <> 1 then
  endgroup
endif;

PROC HA15
preproc
{ HA14 Filter }
if MN5 = 1 then
  skip to HA18
endif;

postproc
if HA15 <> 1 then
  skip to HA18
endif;

PROC HA17
 skip to HA19


PROC HA19
alphavar = $;
if alphachk("ABCDEFGHY") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "No one" is only response }
if pos("Y",HA19) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
  endif;
endif;



PROC HA20
if HA20 <> 1 then
  endgroup
endif;
PROC HA21
alphavar = $;
if alphachk("ABCDEFGHY") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "No one" is only response }
if pos("Y",HA21) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
  endif;
endif;


PROC UF1
preproc
UF1 = HH1;
UF2 = HH2;

PROC UF4
preproc
if addmode then
  display(concat("  i  5 i ",edit("99", LN)));
endif;

postproc
if UF4 <> LN then
  errmsg(2000,"",LN);
  reenter
endif;

PROC UF6
if HL8(LN) <> UF6 then
  errmsg(2004,UF6,HL8(LN));
  { Critical var; enforce consistency during editing }
  reenter;
endif;

PROC UF8Y
recode  UF8Y  :     UF8M        : UF8D => err;
    2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
    2005-2006 : 4,6,9,11        : 1-30 => 0;
    2005-2006 : 2               : 1-28 => 0;
              :                 :      => 1;
endrecode;
if err then
  errmsg(0016);
  reenter
endif;
dd = (HH5Y*100 + HH5M)*100 + HH5D;
ddw= (UF8Y*100 + UF8M)*100 + UF8D;
ss = sysdate("yyyymmdd");
if ddw >= ss or ddw < dd then
  errmsg(0016);
  reenter
endif;
dich = cmcode(UF8M,UF8Y);

PROC UF9
if UF9 <> 1 then
  if endmess() then
    reenter
  endif;
  endlevel
endif;

PROC UF10Y
udob = dich;
ldob = dich - chhlth*12 - 11;
if UF10Y < 9997 then
  ldob = setlb(UF10M,UF10Y,0);
  udob = setub(UF10M,UF10Y,9999);
  if udob < dich - chhlth*12 - 11 or ldob > dich then
    errmsg(2001);
  { reenter }
  endif;
endif;

PROC UF11
if UF11 <> HL5(UF4) then
  errmsg(2002,UF11,HL5(LN));
endif;
{ Lower bound of CMC date of birth }
t = adjlba(ldob,udob,dich,dich,UF11);
if t < 0 then
  errmsg(2003);
{ reenter }
else
  ldob = t
endif;
{ Upper bound of CMC date of birth }
t = adjuba(ldob,udob,dich,dich,UF11);
if t < 0 then
  errmsg(2003);
{ reenter }
else
  udob = t
endif;

PROC BR1
 if BR1 = 1 then
  skip to BR6
endif;

PROC BR2
if BR2 = 1 then
  skip to BR6
elseif BR2 <> 2 then
  skip to BR4
endif;

PROC BR6
{filer BR5}
preproc
if UF11 < 3 then
 endgroup
endif;
postproc
if BR6 <> 1 then
  endgroup
endif;

PROC CE3
alphavar = $;
if alphachk("ABCDY") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
{ Check "No plaything mentionned" is only response }
if pos("Y",CE3) then
  if alphsize > 1 then
    errmsg(9995);
    reenter
    endif;
endif;

PROC CE6
if $ <> 2 then
  endgroup
endif;

PROC CE7
alphavar = $;
if alphachk("ABCX") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;


PROC BF1
if BF1 <> 1 then
  skip to BF2E
endif;

PROC BF2
if BF2 = 1 then
  skip to BF2B
endif;

PROC BF2C
if $ <> 1 then
  skip to BF2E
endif;

PROC BF2D
alphavar = $;
if alphachk("ABCX") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;

PROC BF2E
if $ <> 1 then
  skip to BF3A
endif;

PROC BF2F
alphavar = $;
if alphachk("ABCDX") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
PROC BF3H
{ BF4 Filter }
if BF3H <> 1 then
  endgroup
endif;

PROC FORMCA
if endmess() then
  reenter CA14
endif;

PROC CA14
alphavar = $;
if alphachk("ABCDEFGXYZ") then
  errmsg(9998);
  reenter
else
  $ = alphavar;
endif;
