PROC GLOBAL

{ --------------------------------------------------------------------------- }

set explicit;
{ Working variables }

numeric zscoeff zsr zsb1 zsb2 zsb3 zsb4 zsb5 zsdz zst
  zsdzp zsfx zsll zsul zsa zsknot zsi zsj zsk zsl
  zsz zsc zsy1 zsy2 zsy3 zssd1 zssd2 zshta
  zspmed zsv zshac zshap zshaz zswap zswaz zswac zshtw
  zswhp zswhz zswhc fhaz fwaz fwhz zsflag leapday x
  xdays1 y xdays2 vcheckx addmode qmark lx l totlen
  aok alphsize a maxmem maxovc hhnum minage maxage minab
  chhlth chlabl chlabu mindi cln err i dd ss
  di lev grd numovc j xline xwomen xchild
  ddw diwm udob ldob t ldobfc udobfc ldoblc udoblc
  dich hwsex hwage hwerr birthd birthm
  birthy age xweight xheight 
  rok xmother momhl3 memhl3 mline_valid xfather dadhl3 fline_valid 
  ldofm udofm minam frstrun cnt;

{ --------------------------------------------------------------------------- }

function valid(xvar);
  valid = (!special(xvar) and xvar < 96)
end;

function NAtoZero(xvar);
  if xvar = notappl then
    xvar = 0
  endif;
  NAtoZero = xvar;
end;

{ --------------------------------------------------------------------------- }

{ Function to check all possible combinations of day, month and year }
function vdvalid(vcheckd,vcheckm,vchecky);
  { 0 = no problem, 1 = inconsistency }
  vdvalid = 0;
  { Check day, month and year are consistent }
  recode    vchecky   : vcheckm               : vcheckd    => vcheckx; { !!! }
  2000-2006,9996-9999 : 1,3,5,7,8,10,12,96-99 : 1-31,96-99 => 0;
  2000-2006,9996-9999 : 4,6,9,11              : 1-30,96-99 => 0;
  2000-2006,9996-9999 : 2                     : 1-28,96-99 => 0;
  2000,2004           : 2                     : 29         => 0;       { leap year }
              notappl : notappl               : 66         => 0;
              notappl : notappl               : 44         => 0;
              notappl : notappl               : 0          => 0;
                      :                       :            => 1;
  endrecode;
  if vcheckx then
    vdvalid = 1;
  endif;
end;

{ --------------------------------------------------------------------------- }

PROC MICS3_FF
preproc
maxmem = 50;      { maximum number of household members }
maxovc = 50;      { maximum number of OVCs }
hhnum  = 0;       { last household number used }
minage = 15;      { minimum age of eligible women }
maxage = 49;      { maximum age of eligible women }
minab  = 144;     { minimum age at birth in months = 12 years }
minam  = 156;     { minimum age at first marriage = 13 years }
chhlth = 4;       { maximum age at which child is included in child health module - excludes exact age 5 }
chlabl = 5;       { minimum age at which child is included in child labor module - includes exact age 5 }
chlabu = 14;      { maximum age at which child is included in child labor module - excludes exact age 15 }
mindi = 20050401; { replace "20050401" with earliest interview date in yyyymmdd format }

{ frstrun equals 1 if this is the first run for this cluster, 0 otherwise }
frstrun=tonumber(sysparm());
display("  = %d",frstrun);

PROC HH
{ *** Level 1 *** }

{ HOUSEHOLD QUESTIONNAIRE }

{ HOUSEHOLD INFORMATION PANEL }

{ Check HH date of interview is valid }
recode HH5Y  :     HH5M        : HH5D => err;
   2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
   2005-2006 : 4,6,9,11        : 1-30 => 0;
   2005-2006 : 2               : 1-28 => 0;
             :                 :      => 1;
endrecode;
if err then
  errmsg(0016,"",HH5D,HH5M,HH5Y);
endif;
dd = (HH5Y*100 + HH5M)*100 + HH5D;
ss = sysdate("yyyymmdd");
if dd >= ss or dd < mindi then
  errmsg(0016,"",HH5D,HH5M,HH5Y);
endif;
di = cmcode(HH5M,HH5Y);

{ Check region agrees with cluster number }
recode HH1  : HH6 => err;
    1-3    : 1  => 0;
    4-5    : 2  => 0;
    6-7    : 1  => 0;
    8-9    : 2  => 0;
    10     : 1  => 0;
    11-12  : 2  => 0;
    13-18  : 1  => 0;
    19     : 2  => 0;
    20-26  : 1  => 0;
    27     : 2  => 0;
    28-29  : 1  => 0;
    30     : 2  => 0;
    31     : 1  => 0;
    32-33  : 2  => 0;
    34-36  : 1  => 0;
    37     : 2  => 0;
    38     : 1  => 0;
    39-40  : 2  => 0;
    41-42  : 1  => 0;
    43     : 2  => 0;
    44     : 1  => 0;
    45     : 2  => 0;
    46-49  : 1  => 0;
    50     : 2  => 0;
    51-54  : 1  => 0;
    55-57  : 2  => 0;
    58-59  : 1  => 0;
    60     : 2  => 0;
    61-63  : 1  => 0;
    64-65  : 2  => 0;
    66-67  : 1  => 0;
    68     : 2  => 0;
    69     : 1  => 0;
    70-71  : 2  => 0;
    72-73  : 1  => 0;
    74     : 2  => 0;
    75     : 1  => 0;
    76-77  : 2  => 0;
    78-81  : 1  => 0;
    82     : 2  => 0;
    83     : 1  => 0;
    84     : 2  => 0;
    85-86  : 1  => 0;
    87     : 2  => 0;
    88     : 1  => 0;
    89     : 2  => 0;
    90     : 1  => 0;
    91-92  : 2  => 0;
    93     : 1  => 0;
    94     : 2  => 0;
    95-100 : 1  => 0;
           :    => 1;
endrecode;
if err then
  errmsg(0011,HH1,"HH6",HH6);
endif;

{ Check region agrees with cluster number }
recode HH1  : HH7 => err;
    1-5    : 1  => 0;
    6-9    : 5  => 0;
    10-12  : 7  => 0;
    13-19  : 12 => 0;
    20-27  : 14 => 0;
    28-30  : 18 => 0;
    31-33  : 21 => 0;
    34-37  : 23 => 0;
    38-40  : 26 => 0;
    41-43  : 32 => 0;
    44-45  : 35 => 0;
    46-50  : 44 => 0;
    51-57  : 46 => 0;
    58-60  : 48 => 0;
    61-65  : 51 => 0;
    66-68  : 53 => 0;
    69-71  : 56 => 0;
    72-74  : 59 => 0;
    75-77  : 61 => 0;
    78-82  : 63 => 0;
    83-84  : 65 => 0;
    85-87  : 68 => 0;
    88-89  : 71 => 0;
    90-92  : 73 => 0;
    93-94  : 74 => 0;
    95-100 : 80 => 0;
           :    => 1;
endrecode;
if err then
  errmsg(0011,HH1,"HH7",HH7);
endif;

{ IF HOUSEHOLD INTERVIEW COMPLETE }
if HH9 = 1 then

  { Check not more eligible women than HH members }
  if HH12 > HH11 then
    errmsg(0015,HH12,HH14,HH11);
  endif;

  { Check not more interviews than eligible women }
  if HH13 > HH12 then
    errmsg(0014,HH13,HH12);
  endif;

  { Check not more women and under-5s than HH members }
  if HH12+HH14 > HH11 then
    errmsg(0015,HH12,HH14,HH11);
  endif;

  { Check not more interviews than eligible children }
  if HH15 > HH14 then
    errmsg(0013,HH15,HH14);
  endif;

  { HOUSEHOLD LISTING }

  i = 1;
  while i <= noccurs(MODHL_EDT) do {loop through the household members }
    { Head of household must be on first line }
    if frstrun then
      if (HL3(i) = 1) <=> (HL1(i) <> 1) then
        errmsg(0141,i,HL3(i));
      endif;
    endif;
    
    { Kinship Checks }
    if frstrun then
      { Check the spouse of head of household is of opposite sex }
      if HL3(i) = 2 and HL4(i) = HL4(1) then
        errmsg(0142,i,HL4(i),HL4(1));
      { Check children of head of household are at least minab/12 years younger }
      elseif HL3(i) = 3 and HL5(1) < HL5(i)+(minab/12) then
        errmsg(0143,minab/12,i,HL5(i),HL5(1));
      { Check grandchildren of head of household are at least minab/6 years younger }
      elseif HL3(i) = 5 and HL5(1) < HL5(i)+(minab/6) then
        errmsg(0145,minab/6,i,HL5(i),HL5(1));
      { Check parents of head of household are at least minab/12 years older }
      elseif HL3(i) = 6 and HL5(1)+(minab/12) > HL5(i) then
        errmsg(0146,minab/12,i,HL5(1),HL5(1));
      endif;
    endif;

    { Check eligible women listed as eligible }
    if HL4(i) = 2 and HL5(i) in minage:maxage then
      if HL6(i) <> HL1(i) then
        errmsg(0101,i,HL4(i),HL5(i),HL6(i));
      endif;
    else
      if HL6(i) <> 0 then
        errmsg(0102,i,HL4(i),HL5(i));
      endif;
    endif;

    { Check child labor eligibility correct }
    if HL5(i) in chlabl:chlabu then
      if HL7(i) = 0 or HL7(i) = HL1(i) then
        errmsg(0121,i,HL4(i),HL5(i),HL7(i));
      elseif HL7(i) > HH11 then
        errmsg(0120,i,HL7(i),HH11);
      endif;
    else
      if HL7(i) <> 0 then
        errmsg(0122,i,HL4(i),HL5(i),HL7(i));
      endif;
    endif;

    { Check eligible children listed as eligible }
    if HL5(i) <= chhlth then
      if HL8(i) = 0 or HL8(i) = HL1(i) then
        errmsg(0131,i,HL4(i),HL5(i),HL8(i));
      elseif HL8(i) > HH11 then
        errmsg(0130,i,HL8(i),HH11);
      endif;
    else
      if HL8(i) <> 0 then
        errmsg(0132,i,HL4(i),HL5(i),HL8(i));
      endif;
    endif;

    { Check that HH member identified by HL10 could be child's mother }
    if HL10(i) >= 1 then
      if valid(HL10(i)) and (HL4(HL10(i)) <> 2 or HL5(HL10(i)) < HL5(i)+(minab/12)) then
        errmsg(0164,i,HL4(HL10(i)),HL5(HL10(i)),"i","HL10",HL10(i));
      endif;
    endif;
    if HL10(i) >= 1 then
      if valid(HL10(i)) and HL10(i) > HH11 then
        errmsg(0161,i,"i",HL10(i),HH11);
      elseif HL10(i) = HL1(i) then
        errmsg(0162,i,"i",HL10(i),HL1(i));
      elseif HL7(i) <> 0 then
        if frstrun and HL10(i) <> HL7(i) then
          errmsg(0163,i,HL10(i),"HL7",HL7(i));
        endif;
      elseif HL8(i) <> 0 then
        if frstrun and HL10(i) <> HL8(i) then
          errmsg(0163,HL10(i),"HL8",HL8(i));
        endif;
      endif;
    endif;
    
    { Check that HH member identified by HL12 could be child's father }
    if HL12(i) >= 1 then
      if valid(HL12(i)) and (HL4(HL12(i)) <> 1 or HL5(HL12(i)) < HL5(i)+(minab/12)) then
        errmsg(0164,i,HL4(HL12(i)),HL5(HL12(i)),"i","HL12",HL12(i));
      endif;
    endif;
    if HL12(i) >= 1 then
      if valid(HL12(i)) and HL12(i) > HH11 then
        errmsg(0161,"",HL12(i),HH11);
      elseif HL12(i) = HL1(i) then
        errmsg(0162,"",HL12(i),HL1(i));
      endif;
    endif;

    { Check that relationship code to mother consistent }
    xmother = HL10(i);  { Line number of household member's mother }
    memhl3 = HL3(i);    { Household member's relationship to head of household }
    mline_valid = 0;    { 1 if mother's relationship to HH head consistent with child's relationship to HH head }
    if xmother <> notappl & xmother <> 0 & xmother <> 99 then
      momhl3 = HL3(xmother);  { Mother of household member's relationship to head of household }
      recode memhl3 : momhl3  => err;{ CHILD                    - MOTHER                              }
                  1 : 6       => 0;  { Head                     - Parent                              }
                  2 : 7       => 0;  { Spouse                   - Parent-in-law                       }
                  3 : 1,2     => 0;  { Son/Daughter             - Head/Spouse                         }
                  4 : 13,15   => 0;  { Son/Daughter In-Law      - Other Relative/Not Related          }
                  5 : 3,4     => 0;  { Grandchild               - Son/Daughter/Son or Daughter In-law }
                  6 : 13      => 0;  { Parent                   - Other Relative (Grandparent)        }
                  7 : 15      => 0;  { Parent-In-Law            - Not Related                         }
                  8 : 6       => 0;  { Brother/Sister           - Parent                              }
                  9 : 15      => 0;  { Brother/Sister-In-Law    - Not Related                         }
                 10 : 13      => 0;  { Uncle/Aunt               - Other Relative                      }
                 11 : 8       => 0;  { Niece/Nephew by blood    - Brother or Sister                   }
                 12 : 9       => 0;  { Niece/Nephew by marriage - Brother or Sister-In-Law            }
                 13 : 13      => 0;  { Other Relative           - Other Relative                      }
                 14 : 2,13-15 => 0;  { Adopted/Foster/Stepchild - Spouse/Other Relative/Not related   }
                 15 : 15      => 0;  { Not Related              - Not Related                         }
                 97 :         => 0;  { Inconsistent             - Any                                 }
                    : 97      => 0;  { Any                      - Inconsistent                        }
              98,99 : 99,98   => 0;  { Unknown                  - Unknown                             }
                    :         => 1;  { All others are incorrect                                       }
      endrecode;
      if err then     { Relationship not OK }
        errmsg(0171,i,memhl3,"'","HL10",xmother,momhl3)
      else
        mline_valid = 1;
      endif;
    endif;

        { For Son/Daughter/Head/Spouse/Grandchild/Sibling, try to find (potential) mother's line number, compare it to HL10 }
    if xmother <> notappl {& mline_valid = 0} & (memhl3 in 1,2,3,8) then
     err = 0;
      { Loop through household members }
      do j = 1 while j <= HH11
        { store relationship to HH head of current HH member }
        momhl3 = HL3(j);
        box memhl3 : momhl3 => rok;
                 1 : 6      => 1;   { Head            - Parent                              }
                 2 : 7      => 1;   { Spouse          - Parent-in-law                       }
                 3 : 1,2    => 1;   { Son/Daughter    - Head/Spouse                         }
                 5 : 3,4    => 1;   { Grandchild      - Son/Daughter/Son or Daughter In-Law } 
                 8 : 6      => 1;   { Brother/Sister  - Parent                              }
                   :        => 0;
        endbox;
        if HL4(j) = 2 & rok then    { Found potential mother }
          if xmother <> j then      { But HL10 doesn't equal potential mother's line number }
           err = 1;
         elseif xmother = j then
           err = 0;
           j = HH11; { break out of while loop }
          endif;
         endif;
      enddo;
      if err then
       errmsg(0172,i,"",xmother,j);
      endif;
    endif;


    { Check that relationship code to father consistent }
    xfather = HL12(i);  { Line number of household member's father }
    memhl3 = HL3(i);    { Household member's relationship to head of household }
    fline_valid = 0;    { 1 if father's relationship to HH head consistent with child's relationship to HH head }
    if xfather <> notappl & xfather <> 0 & xfather <> 99 then
      dadhl3 = HL3(xfather);  { Father of household member's relationship to head of household }
      recode memhl3 : dadhl3  => err;{ CHILD                    - FATHER                             }
                  1 : 6       => 0;  { Head                     - Parent                              }
                  2 : 7       => 0;  { Spouse                   - Parent-in-law                       }
                  3 : 1,2     => 0;  { Son/Daughter             - Head/Spouse                         }
                  4 : 13,15   => 0;  { Son/Daughter In-Law      - Other Relative/Not Related          }
                  5 : 3,4     => 0;  { Grandchild               - Son/Daughter/Son or Daughter In-law }
                  6 : 13      => 0;  { Parent                   - Other Relative (Grandparent)        }
                  7 : 15      => 0;  { Parent-In-Law            - Not Related                         }
                  8 : 6       => 0;  { Brother/Sister           - Parent                              }
                  9 : 15      => 0;  { Brother/Sister-In-Law    - Not Related                         }
                 10 : 13      => 0;  { Uncle/Aunt               - Other Relative                      }
                 11 : 8       => 0;  { Niece/Nephew by blood    - Brother or Sister                   }
                 12 : 9       => 0;  { Niece/Nephew by marriage - Brother or Sister-In-Law            }
                 13 : 13      => 0;  { Other Relative           - Other Relative                      }
                 14 : 2,13-15 => 0;  { Adopted/Foster/Stepchild - Spouse/Other Relative/Not related   }
                 15 : 15      => 0;  { Not Related              - Not Related                         }
                 97 :         => 0;  { Inconsistent             - Any                                 }
                    : 97      => 0;  { Any                      - Inconsistent                        }
              98,99 : 99,98   => 0;  { Unknown                  - Unknown                             }
                    :         => 1;  { All others are incorrect                                       }
      endrecode;
      if err then     { Relationship not OK }
        errmsg(0171,i,memhl3,"","HL12",xfather,dadhl3)
      else
        fline_valid = 1;
      endif;
    endif;

    { For Son/Daughter/Head/Spouse/Grandchild/Sibling, try to find (potential) father's line number, compare it to HL12 }
    if xfather <> notappl {& fline_valid = 0} & (memhl3 in 1,2,3,8) then
      { Loop through household members }
      do j = 1 while j <= HH11
        { store relationship to HH head of current HH member }
        dadhl3 = HL3(j);
        box memhl3 : dadhl3 => rok;
                 1 : 6      => 1;   { Head            - Parent                              }
                 2 : 7      => 1;   { Spouse          - Parent-in-law                       }
                 3 : 1,2    => 1;   { Son/Daughter    - Head/Spouse                         }
                 5 : 3,4    => 1;   { Grandchild      - Son/Daughter/Son or Daughter In-Law } 
                 8 : 6      => 1;   { Brother/Sister  - Parent                              }
                   :        => 0;
        endbox;
        if HL4(j) = 1 & rok then    { Found potential father }
          if xfather <> j then      { But HL12 doesn't equal potential father's line number }
            errmsg(0172,i,"",xfather,j)
          endif;
          j = HH11; { break out of while loop }
        endif;
      enddo;
    endif;

    i = i + 1
  enddo;

  { Compare household count variables to recounts of household members }
  cnt = count(MODHL_EDT where HL6 > 0);
  if TOHL6 <> cnt then
    errmsg(0110," 15-49","T0HL6",TOHL6,cnt);
  elseif TOHL6 <> HH12 then
    errmsg(0111," 15-49",TOHL6,"HH12",HH12);
  endif;

  cnt = count(MODHL_EDT where HL7 > 0);
  if TOHL7 <> count(MODHL_EDT where HL7 > 0) then
    errmsg(0110," 5-14","T0HL7",TOHL7,cnt);
  endif;

  cnt = count(MODHL_EDT where HL8 > 0);
  if TOHL8 <> count(MODHL_EDT where HL8 > 0) then
    errmsg(0110,"  5 ","T0HL8",TOHL8,cnt);
  elseif TOHL8 <> HH14 then
    errmsg(0111,"  5 ",TOHL8,"HH14",HH14);
  endif;

  { EDUCATION MODULE }

  i = 1;
  while i <= noccurs(MODED_EDT) do {loop through the education module }

    { Check consistency of highest level and grade }
    if ED3A(i) <> notappl then
      lev = ED3A(i);
      grd = ED3B(i);
      recode lev  : grd         => err; { Fix maximum grade for each possible level }
                  : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
                0 : 0-6         => 0;   { Preschool}
                1 : 0-6         => 0;   { Primary }
                2 : 0-12        => 0;   { Secondary }
                3 : 0-6         => 0;   { Higher }
                6 : 0-6         => 0;   { Non-standard curriculum }
                8 :             => 0;   { DK level }
                9 :             => 0;   { Missing level }
                  :             => 1;
      endrecode;
      if err then
        errmsg(0090,i,"ED3A",lev,"ED3B",grd);
      endif;
    endif;

    { Check current level not higher than highest level }
    if ED6A(i) <= 3 and ED3A(i) <= 3 and ED6A(i) > ED3A(i) then
      errmsg(0091,i,ED6A(i),ED3A(i));
    endif;

    { Check consistency of current level and grade }
    if ED6A(i) <> notappl then
      lev = ED6A(i);
      grd = ED6B(i);
      recode  lev : grd         => err; { Fix maximum grade for each possible level }
                  : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
                0 : 1-6         => 0;   { Preschool}
                1 : 1-6         => 0;   { Primary }
                2 : 1-12        => 0;   { Secondary }
                3 : 1-6         => 0;   { Higher }
                6 : 1-6         => 0;   { Non-standard curriculum }
                8 :             => 0;   { DK level }
                9 :             => 0;   { Missing level }
                  :             => 1;
      endrecode;
      if err then
        errmsg(0090,i,"ED6A",lev,"ED6B",grd);
      endif;
    endif;
    
    { Check current grade not higher than highest grade }
    if (ED3A(i) = ED6A(i)) and valid(ED6B(i)) and valid(ED3B(i)) and ED6B(i) > ED3B(i)+2 then
      errmsg(0092,i,ED6B(i),ED3B(i));
    endif;

    { Check previous year's level not higher than highest level }
    if ED8A(i) <= 3 and ED3A(i) <= 3 and ED8A(i) > ED3A(i) then
      errmsg(0093,i,ED8A(i),ED3A(i));
    endif;

    { Check consistency of highest level and grade }
    if ED8A(i) <> notappl then
      lev = ED8A(i);
      grd = ED8B(i);
      recode  lev : grd         => err; { Fix maximum grade for each possible level }
                  : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
                0 : 1-6         => 0;   { Preschool}
                1 : 1-6         => 0;   { Primary }
                2 : 1-12        => 0;   { Secondary }
                3 : 1-6         => 0;   { Higher }
                6 : 1-6         => 0;   { Non-standard curriculum }
                8 :             => 0;   { DK level }
                9 :             => 0;   { Missing level }
                  :             => 1;
      endrecode;
      if err then
        errmsg(0090,i,"ED8A",lev,"ED8B",grd);
      endif;
    endif;
    
    { Check previous year's grade not higher than highest grade }
    if (ED3A(i) = ED8A(i)) and valid(ED8B(i)) and valid(ED3B(i)) and ED8B(i) > ED3B(i)+2 then
      errmsg(0094,i,ED8B(i),ED3B(i));
    endif;

    i = i + 1;
  enddo;

  { WATER AND SANITATION MODULE }

  { Check WS3's value divisible by 0 or 5 }
  if frstrun and WS3 < 995 and WS3%10 <> 0 and WS3%10 <> 5 then
    errmsg(9993,"WS3",WS3);
  endif;

  { CHILD LABOR MODULE }

  i = 1;
  while i <= noccurs(MODCL_EDT) do {loop through the education module }

      { Check total hours worked not greater than 80 }
      if frstrun and (NAtoZero(CL4(i)) + NAtoZero(CL7(i)) + NAtoZero(CL9(i)) > 80) then
        errmsg(0201,i,CL4(i),CL7(i),CL9(i));
      endif;

    i = i + 1;
  enddo;

  { CHILD DISCIPLINE MODULE }

  { For households with children aged 2-14 }
  if count(MODHL_EDT where HL5 >= 2 AND HL5 <=14) then
  
    { Check count of kids 2-14 correct }
    cnt = count(MODHL_EDT where HL5 >= 2 AND HL5 <=14);
    if frstrun and (CD7 <> count(MODHL_EDT where HL5 >= 2 AND HL5 <=14)) then
      errmsg(0301,CD7,cnt);
    endif;

    { Check selected child not incorrect }
    if frstrun and CD9 > CD7 then
      errmsg(0302,CD9,CD7);
    elseif frstrun and CD7 = 1 and CD9 <> 1 then
      errmsg(0303);
    endif;

    { Check line number of selected child }
    if valid(CD11) then
      if CD11 > HH11 then
        errmsg(0304,CD11,HH11);
      elseif HL5(CD11) > 14 or HL5(CD11) < 2 then
        errmsg(0305,HL5(CD11));
      endif;
    endif;

  endif;



endif; { END HOUSEHOLD INTERVIEW COMPLETE }

PROC WMCH
{ *** Level 2 *** }

{ WOMEN'S QUESTIONNAIRE }

{ WOMEN'S INFORMATION PANEL }

{ Check date of interview }
if WM6D <> notappl then
  recode WM6Y   :     WM6M        : WM6D => err;
      2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
      2005-2006 : 4,6,9,11        : 1-30 => 0;
      2005-2006 : 2               : 1-28 => 0;
                :                 :      => 1;
  endrecode;
  if err then
    errmsg(0016,"i",WM6D,WM6M,WM6Y);
  endif;
  dd = (HH5Y*100 + HH5M)*100 + HH5D;
  ddw= (WM6Y*100 + WM6M)*100 + WM6D;
  ss = sysdate("yyyymmdd");
  if ddw >= ss or ddw < dd then
    errmsg(0016,"i",WM6D,WM6M,WM6Y);
  endif;
  diwm = cmcode(WM6M,WM6Y);
endif;

{ IF WOMEN'S INTERVIEW COMPLETE }
if WM7 = 1 then

  { Check date of birth against date of interview }
  udob = diwm - minage*12;
  ldob = diwm - maxage*12 - 11;
  if WM8Y < 9997 then
    ldob = setlb(WM8M,WM8Y,0);
    udob = setub(WM8M,WM8Y,9999);
    if udob < diwm - maxage*12 - 11 or ldob > diwm - minage*12 then
      if (WM6M = WM8M) and (WM8Y - 50 = WM6Y) then
        if frstrun then
          errmsg(1001,WM6M,WM6Y,WM8M,WM8Y);
        endif;
      else
        errmsg(1001,WM6M,WM6Y,WM8M,WM8Y);
      endif;
    endif;
  endif;

  { Check age against age in household }
  if frstrun and WM9 <> HL5(WM4) then
    errmsg(1002,WM9,HL5(LN));
  endif;

  { Check age, date of birth and date of interview }
  { Lower bound of CMC date of birth }
  t = adjlba(ldob,udob,diwm,diwm,WM9);
  if t < 0 then
    if (WM6M = WM8M) and (WM8Y - WM6Y = WM9 + 1) then
      if frstrun then
        errmsg(1003,WM9,WM8M,WM8Y,WM6M,WM6Y);
      endif;
    else
      errmsg(1003,WM9,WM8M,WM8Y,WM6M,WM6Y);
    endif;
  else
    ldob = t
  endif;
  { Upper bound of CMC date of birth }
  t = adjuba(ldob,udob,diwm,diwm,WM9);
  if t < 0 then
    if (WM6M = WM8M) and (WM8Y - WM6Y = WM9 + 1) then
      if frstrun then
        errmsg(1003,WM9,WM8M,WM8Y,WM6M,WM6Y);
      endif;
    else
      errmsg(1003,WM9,WM8M,WM8Y,WM6M,WM6Y);
    endif;
  else
    udob = t
  endif;

  if frstrun then
    { Check school attendance against value in household }
    if WM10 <> ED2(LN) and ED3(LN) <> 0 then
      errmsg(1011,ED2(LN),WM10);
    endif;
    
    { If ever attended school }
    if WM10 = 1 then
      { Check highest level against value in household }
      if WM11 <> ED3A(LN) then
        errmsg(1012,ED3A(LN),WM11);
      endif;

      { Check highest grade against value in household }
      if WM12 <> ED3B(LN) then
        errmsg(1013,ED3B(LN),WM12);
      endif;
    endif;
  endif;
  
  { If ever attended school }
  if WM10 = 1 then
    { Check highest level and grade consistent }
    lev = WM11;
    grd = WM12;
    recode lev  : grd         => err; { Fix maximum grade for each possible level }
                : 97,98,99    => 0;   { Inconsistent, missing, DK grade }
              0 : 0-6         => 0;   { Preschool}
              1 : 0-6         => 0;   { Primary }
              2 : 0-12        => 0;   { Secondary }
              3 : 0-6         => 0;   { Higher }
              6 : 0-6         => 0;   { Non-standard curriculum }
              8 :             => 0;   { DK level }
              9 :             => 0;   { Missing level }
                :             => 1;
    endrecode;
    if err then
      errmsg(1014,WM11,WM12);
    endif;
  endif;
    
  { CHILD MORTALITY MODULE }

  { If ever given birth }
  if CM1 = 1 then
  
    { Check date of first birth against mother's age }
    ldobfc = ldob + minab;
    udobfc = diwm;
    if CM2AY < 9997 then
      ldobfc = setlb(CM2AM,CM2AY,0);
      udobfc = setub(CM2AM,CM2AY,9999);
      if udobfc < ldob + minab then
        errmsg(1101,CM2AM,CM2AY,int(minab/12));
      elseif ldobfc > diwm then
        errmsg(1102,CM2AM,CM2AY,WM6M,WM6Y);
      endif;
    endif;

    { Check years since first birth against age }
    if CM2AY > 9996 and WM9 - CM2B < (minab/12) then
      errmsg(1103,minab/12,WM9,CM2B);
    endif;

    { Check for at least one child living at home - otherwise CM3 must be 2 }
    if CM3 = 1 and CM4A = 0 and CM4B = 0 then
      errmsg(1110,"CM3=","CM4A+CM4B");
    endif;

    { Check for at least one child living away - otherwise CM5 must be 2 }
    if CM5 = 1 and CM6A = 0 and CM6B = 0 then
      errmsg(1110,"CM5=","CM6A+CM6B");
    endif;

    { Check for at least one child who died - otherwise CM7 must be 2 }
    if CM7 = 1 and CM8A = 0 and CM8B = 0 then
      errmsg(1110,"CM7=","CM8A+CM8B");
    endif;

    { Check count of children ever born with total }
    x = NAtoZero(CM4A)+NAtoZero(CM4B)+
        NAtoZero(CM6A)+NAtoZero(CM6B)+
        NAtoZero(CM8A)+NAtoZero(CM8B);
    if x <> CM9 then
      errmsg(1111,CM9,x);
    elseif CM9 = 0 then
      errmsg(1110,"CM4+CM6+CM8>=","CM9");
    endif;

    { Check date of last birth against mother's age }
    ldoblc = ldobfc;
    udoblc = diwm;
    if CM11Y < 9997 then
      ldoblc = setlb(CM11M,CM11Y,0);
      udoblc = setub(CM11M,CM11Y,9999);
      if udoblc < ldob + minab then
        errmsg(1121,CM11M,CM11Y,int(minab/12));
      elseif ldoblc > diwm then
        errmsg(1122,CM11M,CM11Y,WM6M,WM6Y);
      endif;
    endif;

    { Check data of first birth against date of last birth }
    if CM9 = 1 then
      if (CM11Y <> CM2AY or CM11M <> CM2AM or CM11D <> CM2AD) then
        errmsg(1123,CM2AD,CM2AM,CM2AY,CM11D,CM11M,CM11Y);
      endif;
    else
      if ldobfc > udoblc then
        errmsg(1124,CM11D,CM11M,CM11Y,CM2AD,CM2AM,CM2AY);
      endif;
    endif;

    { Check if birth in last 24 months }
    { Note use of parentheses for correct check }
    if CM11Y = WM6Y or CM11Y = WM6Y-1 or (CM11Y = WM6Y-2 and
     ((CM11M > WM6M and CM11M <= 12)  or (CM11M = WM6M and
       CM11D > WM6D and CM11D <= 31))) then
      if CM12 <> "Y" then
        errmsg(1131,CM11D,CM11M,CM11Y,WM6D,WM6M,WM6Y,CM12);
      endif;
    else
      if CM12 <> "N" then
        errmsg(1132,CM11D,CM11M,CM11Y,WM6D,WM6M,WM6Y,CM12);
      endif;
    endif;

  endif;

  { MATERNAL AND NEWBORN HEALTH MODULE }

  { Check special answers }
  if MN10 = 1 and (MN11A = 9 <=> MN11 < 9997) then
    errmsg(1301,MN11A,MN11);
  endif;

  { Check consistency between units and numbers }
  if MN12 = 1 then
    recode MN13U  : MN13N       => err;
              0   : 0           => 0;   { Immediately }
              1   : 0-23        => 0;   { Hours }
              2   : 1-90        => 0;   { Days }
              9   : 98-99       => 0;   { DK, Missing }
                  :             => 1;
    endrecode;
    if err then
      errmsg(9992,"MN13U",MN13U,"MN13N",MN13N);
    endif;
  endif;

  { MARRIAGE/UNION MODULE }

  { Check woman at least 0 and 4 when married (minam=min. age at marriage) }
  if frstrun then
    if MA6Y < 9996 and WM8Y < 9996 then
      if valid(MA6M) and valid(WM8M) then
        if cmcode(MA6M,MA6Y) - cmcode(WM8M,WM8Y) < 0 then
          errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,0);
        elseif cmcode(MA6M,MA6Y) - cmcode(WM8M,WM8Y) < minam then
          errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,minam/12);
        endif;
      else
        if MA6Y - WM8Y < 0 then
          errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,0);
        elseif MA6Y - WM8Y < int(minam/12)-1 then
          errmsg(1401,WM8M,WM8Y,MA6M,MA6Y,minam/12);
        endif;
      endif;
    endif;
  endif;

  { Check date of first marriage consistent with age at first marriage }
  if MA6Y < 9997 and valid(MA8) then
    ldofm = setlb(1,MA6Y,0);
    udofm = setub(12,MA6Y,9999);
    { Lower bound of CMC date of first marriage }
    t = adjlba(ldob,udob,ldofm,udofm,MA8);
    if t < 0 then
      errmsg(1403,MA8,MA6M,MA6Y,WM8M,WM8Y);
    endif;
    { Upper bound of CMC date of first marriage }
    t = adjuba(ldob,udob,ldofm,udofm,MA8);
    if t < 0 then
      errmsg(1403,MA8,MA6M,MA6Y,WM8M,WM8Y);
    endif;
  endif;

  { Check age at first marriage against age }
  if valid(MA8) and MA8 > WM9 then
    errmsg(1404,MA8,WM9);
  endif;

  { CONTRACEPTION MODULE }

  { Check units and number consistent }
  if CP4A = 1 then
    recode CP4CU  : CP4CN       => err;
              1,2 : 0-90        => 0;   { Months, Years }
              9   : 93-96,98,99 => 0;   { Special, DK, Missing }
                  :             => 1;
    endrecode;
    if err then
      errmsg(9992,"CP4CU",CP4CU,"CP4CN",CP4CN);
    endif;
  endif;


endif; { END WOMEN'S INTERVIEW COMPLETE }

{ CHILDREN'S QUESTIONNAIRE }

{ UNDER-FIVE CHILD INFORMATION PANEL }

{ Check caretaker's line number against value in household}
if UF6 <> notappl and HL8(LN) <> UF6 then
  errmsg(2004,UF6,LN,HL8(LN));
endif;

{ Check date of interview consistent }
if UF8D <> notappl then
  recode  UF8Y  :     UF8M        : UF8D => err;
      2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
      2005-2006 : 4,6,9,11        : 1-30 => 0;
      2005-2006 : 2               : 1-28 => 0;
                :                 :      => 1;
  endrecode;
  if err then
    errmsg(0016,"",UF8D,UF8M,UF8Y);
  endif;
  dd = (HH5Y*100 + HH5M)*100 + HH5D;
  ddw= (UF8Y*100 + UF8M)*100 + UF8D;
  ss = sysdate("yyyymmdd");
  if ddw >= ss or ddw < dd then
    errmsg(0016,"",UF8D,UF8M,UF8Y);
  endif;
  dich = cmcode(UF8M,UF8Y);
endif;

{ IF CHILD'S INTERVIEW COMPLETE }
if UF9 = 1 then

  { Check date of birth consistent with date of interview }
  udob = dich;
  ldob = dich - chhlth*12 - 11;
  if UF10Y < 9997 then
    ldob = setlb(UF10M,UF10Y,0);
    udob = setub(UF10M,UF10Y,9999);
    if udob < dich - chhlth*12 - 11 or ldob > dich then
      if (UF8M = UF10M) and (UF8Y - 5 = UF10Y) then
        { Do nothing if DOB valid given day }
        if valid(UF10d) and UF8D <= UF10D then
        { Error if DOB invalid given day }
        elseif valid(UF8D) and UF8D > UF10D then
          errmsg(2001,UF10M,UF10Y,UF8M,UF8Y);
        { If impossible to verify DOB since UF8D invalid, warn on first run }
        elseif !valid(UF8D) and frstrun then
          errmsg(2001,UF10M,UF10Y,UF8M,UF8Y);
        endif;
      else
        errmsg(2001,UF10M,UF10Y,UF8M,UF8Y);
      endif;
    endif;
  endif;

  { Check age consistent with value in household }
  if frstrun and UF11 <> HL5(UF4) then
    errmsg(2002,UF11,HL5(LN));
  endif;

  { Check age, date of birth and date of interview consistent }
  { If day, month and year of birth are valid }
  if valid(UF10D) and valid(UF10M) and UF10Y < 9997 then
    if (UF10M = UF8M and UF10D > UF8D) | (UF10M > UF8M) then
      age = UF8Y - UF10Y - 1;
    else
      age = UF8Y - UF10Y;
    endif;
    if age <> UF11 then
      errmsg(2003,UF11,UF10D,UF10M,UF10Y,UF8D,UF8M,UF8Y);
    endif;
  else
    { Lower bound of CMC date of birth }
    t = adjlba(ldob,udob,dich,dich,UF11);
    if t < 0 then
      if (UF8M = UF10M) and (UF8Y - 5 = UF10Y) then
        if frstrun then
          errmsg(2003,UF11,UF10D,UF10M,UF10Y,UF8D,UF8M,UF8Y);
        endif;
      else
        errmsg(2003,UF11,UF10D,UF10M,UF10Y,UF8D,UF8M,UF8Y);
      endif;
    else
      ldob = t
    endif;
    { Upper bound of CMC date of birth }
    t = adjuba(ldob,udob,dich,dich,UF11);
    if t < 0 then
      if (UF8M = UF10M) and (UF8Y - 5 = UF10Y) then
        if frstrun then
          errmsg(2003,UF11,UF10D,UF10M,UF10Y,UF8D,UF8M,UF8Y);
        endif;
      else
        errmsg(2003,UF11,UF10D,UF10M,UF10Y,UF8D,UF8M,UF8Y);
      endif;
    else
      udob = t
    endif;
  endif;
endif; { END CHILD'S INTERVIEW COMPLETE }
